@@ -46,16 +46,22 @@ passed through:
  volumeSize  -- Size of the EBS volume (in gigs).
  deleteOnTermination -- Whether this EBS will be deleted when the
                 instance terminates.
+ volumeType  -- The volume type, one of "standard" or "iol"
+ iops        -- The number of I/O operations per second that the volume
+                supports, an integer between 1 and 1000. Only valid for
+                volumes of type "iol".
 
 =head1 STRING OVERLOADING
 
 When used in a string context, this object will be interpolated as:
 
- deviceName=snapshotId:volumeSize:deleteOnTermination
+ deviceName=snapshotId:volumeSize:deleteOnTermination:volumeType:iops
+
+The :iops portion is only valid when the volumeType is "iol".
 
 e.g.
 
- /dev/sdg=snap-12345:20:true
+ /dev/sdg=snap-12345:20:true:standard
 
 This happens to be the same syntax used to specify block device
 mappings in run_instances(). See L<VM::EC2>.
@@ -108,12 +114,16 @@ sub ebs {
 sub snapshotId { shift->ebs->snapshotId }
 sub volumeSize { shift->ebs->volumeSize }
 sub deleteOnTermination { shift->ebs->deleteOnTermination }
+sub volumeType  { shift->ebs->volumeType }
+sub iops        { shift->ebs->iops       }
 
 sub as_string {
-    my $self = shift;
-    my $dot  = $self->deleteOnTermination ? 'true' : 'false';
+    my $self  = shift;
+    my $dot   = $self->deleteOnTermination ? 'true' : 'false';
+    my $vtype = $self->volumeType;
+    my @type_iops = $vtype eq 'iol' ? ($vtype,$self->iops) : $vtype;
     return $self->deviceName.'='.
-	join ':',$self->snapshotId,$self->volumeSize,$dot;
+	join ':',$self->snapshotId,$self->volumeSize,$dot,@type_iops;
 }
 
 1;
